/****************************************************************************
  GBAtile.c                                                                        **
  
  load a pcx image into memory                                            
  output a c file of that picture suitable for loading into GBA  
****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/*PCX Header structure*/
struct PCXHeader {
  char Manufacturer, Version, Encoding, BitsPerPixel;
  short int xMin, yMin, xMax, yMax;
  char Other[116];
} Header;

unsigned int pic [64000];
unsigned char pal [256*3];

int size;

// 0 = no, 1 = yes
int b16color = 1;

int LoadPCX (char *FileName)
/*
	This loads in the actual PCX-file and displays it.
*/
{
	
  /*File handle*/
	FILE *PCXFile;
	/*Required for decoding*/
	unsigned char DataByte, HowMany;
	int x, y;
	int nextx;
  int c,pos;

	/*Check to see whether the file exists and can be opened*/
	PCXFile = fopen (FileName, "rb");
	if (PCXFile == NULL)
	{
    // hm, we can't open the file for some reason
  	printf("Wrong version: %s\n",FileName);
    return -1;
	}

	/*Read in header information*/
	fread (&Header, 128, 1, PCXFile);
	/*Check to see whether we can display it*/
	if (Header.Version != 5) 
  {
	  /*If other version than 5 don't display*/
	  printf("Wrong version: %s\n",FileName);
	  fclose (PCXFile);
		return -1;
	}
	
  if (Header.xMax>319 || Header.xMin<0 || Header.yMax>200 || Header.yMin<0) 
  {
    /* too big, or too small */
		fclose (PCXFile);
		printf("Picture too big: %s\n",FileName);
		exit(1);
	}

	y=Header.yMin;
	x=Header.xMin;
	Header.xMax++;
	Header.yMax++;
  pos = 0;

  size = (Header.yMax * Header.xMax);
 
  /*Decode and display graphics*/
  while (y<Header.yMax) 
  {
		/*Read next byte*/
		fread (&DataByte, 1, 1, PCXFile);

		/*Reset counter*/
		HowMany=1;

		/*If it is encoded, extract the count information and read in the colour byte*/
		if ((DataByte & 0xC0)==0xC0) {
			HowMany = (DataByte & 0x3F);
			fread (&DataByte, 1, 1, PCXFile);
		}

    for (c = 0;c<HowMany;c++)
    {
      pic[pos] = DataByte;
      pos++;
    }

		nextx = x+HowMany;
		if (nextx>Header.xMax)
		{
			y++;
			nextx -= Header.xMax;
		}
		x=nextx;
	 }

  // read the palette
  fread (&pal[0], 1, 256*3, PCXFile);

  fclose (PCXFile);
  return 1;
}

int output_1_256tile (FILE *out,int x, int y)
{
  int dx;
  int dy;
  int pos;  
 
  for (dy=y*8;dy<(y+1)*8;dy++)
  for (dx=x*8;dx<(x+1)*8;dx++)
  {
    pos = (dy*Header.xMax) + dx;
    fprintf(out,"%d",pic[pos]);
    pos++;
    if ((pos & 7) == 0)
    {
      fprintf (out,"\n");
    } 

    if (pos < size)
      fprintf (out,", ");
  }
}

int output_1_16tile (FILE *out,int x, int y)
{
  int dx;
  int dy;
  int pos;  
 
  for (dy=y*8;dy<(y+1)*8;dy++)
  {
    for (dx=x*8;dx<(x+1)*8;dx = dx+2)
    {
      pos = (dy*Header.xMax) + dx;
      fprintf(out,"%d",(((pic[pos+1] &15)<<4)+(pic[pos] &15)));
      if (pos < size)
        fprintf (out,", ");
    }
    if (dy & 1)
    {
      fprintf (out,"\n");
    } 
  }
}

int output_palette (FILE *out)
{
  int x;
  int c;
  for (x = 0; x <256; x++)
  {
    c = ((pal[(x*3)+2]) >>3) << 10;
    c += ((pal[(x*3)+1]) >>3) << 5;
    c += ((pal[(x*3)]) >>3) ;

    fprintf (out,"%u",c);
    if (x < 255)
      fprintf(out,", ");
    if ((x&7) == 7)
      fprintf (out,"\n");

  }
}

int Output (char *name)
/*
	This loads in the actual PCX-file and displays it.
*/
{
	
  /*File handle*/
	FILE *OutFile;
	char FileName[100];

  int x, y;
	int nextx;
  int c,pos;

  strncpy (FileName,name,80);
  strcat (FileName,".h");

	/*Check to see whether the file exists and can be opened*/
	OutFile = fopen (FileName, "w+b");
	if (OutFile == NULL)
	{
    // hm, we can't open the file for some reason
  	printf("Can't open file: %s_tiles\n",FileName);
    return -1;
	}

  if (b16color)
  {
    fprintf (OutFile, "const u16 %s_size = %d;\n\n", name, size>>1);
    fprintf (OutFile, "const u8 %s_tiles [%d] = {\n", name, size>>1);
  }
  else
  {
    fprintf (OutFile, "const u16 %s_size = %d;\n\n", name, size);
    fprintf (OutFile, "const u8 %s_tiles [%d] = {\n", name, size);
  }
  for (y = 0; y<Header.yMax; y = y+8)
  for (x = 0; x < Header.xMax; x=x+8)
  {
    if (b16color)
      output_1_16tile (OutFile,x>>3,y>>3);
    else
      output_1_256tile (OutFile,x>>3,y>>3);
  }

  fprintf (OutFile, "};\n\n");

  fprintf (OutFile,"const u16 %s_pal [256] = {\n",name);
  output_palette (OutFile);
  fprintf (OutFile,"};\n");

  fclose (OutFile);
  return 1;

}


int main (int argc, char *argv[])
{
  if (argc <2)
  {
    printf ("gbatile image.pcx name\n");
    return -1;
  }
  
  LoadPCX(argv[1]);
 
  Output (argv[2]);
}

